"""Version details for ovrlib"""

__title__ = "ovrlib"
__description__ = "Library to interface with per-state online voter registration APIs"
__url__ = "https://github.com/vote/state-registration-apis"
__version__ = "0.0.5"
__author__ = "Sage Weil"
__author_email__ = "sage@newdream.net"
__license__ = "Apache 2.0"
__maintainer__ = "Sage Weil"
__maintainer_email__ = "sage@newdream.net"
__keywords__ = "python ovr vote"
